; ArrowLISP Example Program
; Copyright (C) 1998-2006 Nils M Holm. All rights reserved.
; See the file LICENSE of the ArrowLISP distribution
; for conditions of use.

; Rename variables of lambda expressions
; (perform alpha convfersion):
; (lv-rename '(lambda (x) (lambda (x) x)))
; => (lambda (x:0) (lambda (x:1) x:1))

(require '=nmath)

(define (lv-rename x)
  (letrec

  ; Rename X to first match of NLIST
  ((rename (lambda (x nlist)
    (let ((nx (assq x nlist)))
      (cond (nx (cdr nx))
        (t nx)))))

  ; Rename each symbol in L
  (rename-list (lambda (l nlist r)
    (cond ((null l) (reverse r))
      (t (rename-list (cdr l) nlist
           (cons (rename (car l) nlist) r))))))

  ; Create a binding of the form
  ; (NAME . UNIQUE-NAME)
  ; Unique names are created by appending :LEVEL
  (bind (lambda (name level)
     (cons name (implode (append (explode name)
                                 '#: level)))))

  ; Create a set of local (unique) names
  ; OLD   - NLIST so far
  ; NEW   - list of symbols to add to NLIST
  ; LEVEL - nesting level LAMBDA
  (localnames (lambda (old new level)
    (cond ((null new) old)
      (t (cons (bind (car new) level)
               (localnames old (cdr new) level))))))

  ; Check whether X is a lambda expression
  (lambda-p (lambda (x)
    (and (eq (car x) 'lambda)
         (not (atom (cdr x)))
         (listp (cadr x))
         (not (atom (cddr x))))))

  ; Re-name all lambda variables of X.
  ; NLIST holds the bindings used to rename symbols.
  ; LEVEL is the current nesting level of LAMBDA.
  (lv-ren (lambda (x nlist level)
    (cond ((null x) ())
      ((atom x) (rename x nlist))
      ((lambda-p x)
        (let ((new-nlist (localnames nlist (cadr x) level)))
          (list 'lambda
            (rename-list (cadr x) new-nlist ())
            (lv-ren (caddr x) new-nlist (+ '#1 level)))))
      (t (cons (lv-ren (car x) nlist level)
               (lv-ren (cdr x) nlist level)))))))

  (lv-ren x () '#0)))
